@@ -22,55 +22,93 @@
 #include <sys/types.h>
 
 /* GLOBALS */
-struct wavemon_conf conf;
+#define MAX_IFLIST_ENTRIES 64
+static char *if_list[MAX_IFLIST_ENTRIES]; /* array of WiFi interface names */
 int conf_items;			/* index into array storing menu items */
-static int if_list;		/* index into array of WiFi interface names */
 
-static void version(void)
+static char *on_off_names[] = { [false] = "Off", [true] = "On", NULL };
+static char *action_items[] = {
+	[TA_DISABLED]	= "Disabled",
+	[TA_BEEP]	= "Beep",
+	[TA_FLASH]	= "Flash",
+	[TA_BEEP_FLASH]	= "Beep+Flash",
+	NULL
+};
+
+static char *sort_order[] = {
+	[SO_CHAN]	= "Channel",
+	[SO_SIGNAL]	= "Signal",
+	[SO_MAC]	= "MAC",
+	[SO_ESSID]	= "Essid",
+	[SO_OPEN]	= "Open",
+	[SO_CHAN_SIG]	= "Chan/Sig",
+	[SO_OPEN_SIG]	= "Open/Sig",
+	NULL
+};
+
+static char *screen_names[] = {
+	[SCR_INFO]	= "Info screen",
+	[SCR_LHIST]	= "Histogram",
+	[SCR_SCAN]	= "Scan window",
+	NULL
+};
+
+struct wavemon_conf conf = {
+	.if_idx			= 0,
+
+	.stat_iv		= 100,
+	.info_iv		= 10,
+	.slotsize		= 4,
+	.meter_decay		= 0,
+
+	.check_geometry		= false,
+	.cisco_mac		= false,
+	.override_bounds	= false,
+
+	.sig_min		= -100,
+	.sig_max		= -10,
+	.noise_min		= -120,
+	.noise_max		= -40,
+
+	.scan_sort_order	= SO_CHAN_SIG,
+	.scan_sort_asc		= false,
+	.lthreshold_action	= TA_DISABLED,
+	.lthreshold		= -80,
+	.hthreshold_action	= TA_DISABLED,
+	.hthreshold		= -10,
+
+	.startup_scr		= 0,
+};
+
+/** Populate interface list */
+void conf_get_interface_list(void)
 {
-	printf("wavemon wireless monitor %s\n", PACKAGE_VERSION);
-	printf("Distributed under the terms of the GPLv3.\n");
+	char *old_if = NULL;
+	int idx;
+
+	for (idx = 0; if_list[idx]; idx++) {
+		if (idx == conf.if_idx)
+			old_if = if_list[idx];
+		else
+			free(if_list[idx]);
+	}
+	iw_get_interface_list(if_list, MAX_IFLIST_ENTRIES);
+	if (!if_list[0])
+		err_quit("no supported wireless interfaces found!");
+
+	conf.if_idx = 0;
+	if (old_if) {
+		idx = argv_find(if_list, old_if);
+		if (idx > 0)
+			conf.if_idx = idx;
+		free(old_if);
+	}
 }
 
-static void usage(void)
+/** Return currently selected interface name */
+const char *conf_ifname(void)
 {
-	printf("Usage: wavemon [ -dhlrv ] [ -i ifname ]\n\n");
-	printf("  -d            Dump the current device status to stdout and exit\n");
-	printf("  -h            This help screen\n");
-	printf("  -i <ifname>   Use specified network interface (default: auto)\n");
-	printf("  -r            Generate random levels (for testing purposes)\n");
-	printf("  -v            Print version number and exit\n\n");
-}
-
-static void getargs(int argc, char *argv[])
-{
-	int arg, tmp;
-
-	while ((arg = getopt(argc, argv, "dhi:rv")) >= 0)
-		switch (arg) {
-		case 'd':
-			dump_parameters();
-			exit(EXIT_SUCCESS);
-		case 'h':
-			usage();
-			exit(EXIT_SUCCESS);
-		case 'i':
-			if ((tmp = ll_scan(if_list, "S", optarg)) >= 0) {
-				strncpy(conf.ifname, ll_get(if_list, tmp),
-					sizeof(conf.ifname));
-				break;
-			}
-			err_quit("no wireless extensions found on '%s'", optarg);
-		case 'r':
-			conf.random = true;
-			break;
-		case 'v':
-			version();
-			exit(EXIT_SUCCESS);
-		default:
-			/* bad argument. bad bad */
-			exit(EXIT_FAILURE);
-		}
+	return if_list[0] && if_list[conf.if_idx] ? if_list[conf.if_idx] : "(none)";
 }
 
 /* Return full path of rcfile. Allocates string which must bee free()-d. */
@@ -91,137 +129,18 @@ static char *get_confname(void)
 	return full_path;
 }
 
-static void read_cf(void)
-{
-	char tmp[0x100], lv[0x20], rv[0x20];
-	struct conf_item *ci = NULL;
-	FILE *fd;
-	size_t len;
-	int lnum, found, v_int;
-	char *lp, *conv_err;
-	char *cfname = get_confname();
-
-	if (access(cfname, F_OK) != 0)
-		goto done;
-
-	fd = fopen(cfname, "r");
-	if (fd == NULL)
-		err_sys("can not read configuration file '%s'", cfname);
-
-	for (lnum = 1; fgets(tmp, sizeof(tmp), fd); lnum++) {
-
-		lp = tmp + strspn(tmp, " ");
-		if (*lp == '#' || *lp == '\n')
-			continue;
-
-		len = strcspn(lp, " =");
-		if (len > sizeof(lv))
-			err_quit("parse error in %s, line %d: identifier too long",
-				 cfname, lnum);
-		strncpy(lv, lp, len);
-		lv[len] = '\0';
-		lp += len;
-
-		ll_reset(conf_items);
-		for (found = 0; !found && (ci = ll_getall(conf_items)); )
-			found = (ci->type != t_sep && ci->type != t_func &&
-				 strcasecmp(ci->cfname, lv) == 0);
-		if (!found) {
-			err_msg("%s, line %d: ignoring unknown identifier '%s'",
-				cfname, lnum, lv);
-			continue;
-		}
-
-		lp += strspn(lp, " ");
-		if (*lp++ != '=')
-			err_quit("parse error in %s, line %d: missing '=' operator in assignment",
-				 cfname, lnum);
-		lp += strspn(lp, " ");
-
-		len = strcspn(lp, " \n");
-		if (len > sizeof(rv))
-			err_quit("parse error in %s, line %d: argument too long", cfname, lnum);
-		else if (*lp == '\n')
-			err_quit("parse error in %s, line %d: argument expected", cfname, lnum);
-		strncpy(rv, lp, len);
-		rv[len] = '\0';
-
-		switch (ci->type) {
-		case t_int:
-			v_int = strtol(rv, &conv_err, 10);
-			if (*conv_err != '\0') {
-				err_quit("parse error in %s, line %d: integer value expected, '%s' found instead",
-					 cfname, lnum, rv);
-			} else if (v_int > ci->max) {
-				err_quit("parse error in %s, line %d: value exceeds maximum of %d",
-					 cfname, lnum, (int)ci->max);
-			} else if (v_int < ci->min) {
-				err_quit("parse error in %s, line %d: value is below minimum of %d",
-					 cfname, lnum, (int)ci->min);
-			} else {
-				*ci->v.i = v_int;
-			}
-			break;
-		case t_string:
-			if (strlen(rv) > ci->max)
-				err_quit("parse error in %s, line %d: argument too long (max %d chars)",
-					 cfname, lnum, ci->max);
-			strncpy(ci->v.s, rv, LISTVAL_MAX);
-			break;
-		case t_switch:
-			if (!strcasecmp(rv, "on") || !strcasecmp(rv, "yes") ||
-			    !strcasecmp(rv, "enabled") || !strcasecmp(rv, "1")) {
-				*(ci->v.b) = 1;
-			} else if (!strcasecmp(rv, "off") || !strcasecmp(rv, "no") ||
-				   !strcasecmp(rv, "disabled") || !strcasecmp(rv, "0")) {
-				*(ci->v.b) = 0;
-			} else {
-				err_quit("parse error in %s, line %d: boolean expected, '%s' found instead",
-					 cfname, lnum, rv);
-			}
-			break;
-		case t_list:
-			v_int = ll_scan(ci->list, "S", rv);
-			if (v_int < 0)
-				err_quit("parse error in %s, line %d: '%s' is not a valid argument here",
-					 cfname, lnum, rv);
-			*ci->v.b = v_int;
-			break;
-		case t_listval:
-			v_int = ll_scan(ci->list, "S", rv);
-			if (v_int < 0)
-				err_quit("parse error in %s, line %d: '%s' is not a valid argument here",
-					 cfname, lnum, rv);
-			strncpy(ci->v.s, ll_get(ci->list, v_int), LISTVAL_MAX);
-			break;
-		case t_sep:	/* These two cases are missing from the enum, they are not handled */
-		case t_func:	/* To pacify gcc -Wall, fall through here */
-			break;
-		}
-	}
-	fclose(fd);
-done:
-	free(cfname);
-}
-
 static void write_cf(void)
 {
 	char tmp[0x100], rv[0x40];
 	struct conf_item *ci = NULL;
 	char *lp, *cp;
 	int add, i;
-	FILE *fd;
 	char *cfname = get_confname();
 	int cfld = ll_create();
+	FILE *fd = fopen(cfname, "w");
 
-	if (access(cfname, F_OK) == 0) {
-		fd = fopen(cfname, "r");
-		if (fd == NULL)
-			err_sys("can not read configuration file '%s'", cfname);
-		while (fgets(tmp, sizeof(tmp), fd))
-			ll_push(cfld, "s", tmp);
-		fclose(fd);
-	}
+	if (fd == NULL)
+		err_sys("failed to open configuration file '%s'", cfname);
 
 	for (ll_reset(conf_items); (ci = ll_getall(conf_items)); ) {
 		if (ci->type != t_sep && ci->type != t_func &&
@@ -230,18 +149,12 @@ static void write_cf(void)
 			case t_int:
 				sprintf(rv, "%d", *ci->v.i);
 				break;
-			case t_string:	/* fall through */
-			case t_listval:
-				strcpy(rv, ci->v.s);
-				break;
-			case t_switch:
-				sprintf(rv, "%s", *ci->v.b ? "on" : "off");
-				break;
 			case t_list:
-				sprintf(rv, "%s", (char *)ll_get(ci->list, *ci->v.b));
+				if (!argv_count(ci->list))
+					continue;
+				sprintf(rv, "%s", ci->list[*ci->v.i]);
 				str_tolower(rv);
 				break;
-				/* Fall through, the rest are dummy statements to pacify gcc -Wall */
 			case t_sep:
 			case t_func:
 				break;
@@ -279,10 +192,6 @@ static void write_cf(void)
 		}
 	}
 
-	fd = fopen(cfname, "w");
-	if (fd == NULL)
-		err_sys("can not write to configuration file '%s'", cfname);
-
 	for (ll_reset(cfld); (lp = ll_getall(cfld)); )
 		fputs(lp, fd);
 	fclose(fd);
@@ -291,6 +200,108 @@ static void write_cf(void)
 	free(cfname);
 }
 
+static void read_cf(void)
+{
+	char tmp[0x100], lv[0x20], rv[0x20];
+	struct conf_item *ci = NULL;
+	FILE *fd;
+	size_t len;
+	int lnum, found, v_int;
+	char *lp, *conv_err;
+	bool file_needs_update = false;
+	char *cfname = get_confname();
+
+	if (access(cfname, F_OK) != 0)
+		goto done;
+
+	fd = fopen(cfname, "r");
+	if (fd == NULL)
+		err_sys("can not read configuration file '%s'", cfname);
+
+	for (lnum = 1; fgets(tmp, sizeof(tmp), fd); lnum++) {
+		lp = tmp + strspn(tmp, " ");
+		if (*lp == '#' || *lp == '\n')
+			continue;
+
+		len = strcspn(lp, " =");
+		if (len > sizeof(lv))
+			err_quit("parse error in %s, line %d: identifier too long",
+				 cfname, lnum);
+		strncpy(lv, lp, len);
+		lv[len] = '\0';
+		lp += len;
+
+		ll_reset(conf_items);
+		for (found = 0; !found && (ci = ll_getall(conf_items)); )
+			found = (ci->type != t_sep && ci->type != t_func &&
+				 strcasecmp(ci->cfname, lv) == 0);
+		if (!found) {
+			err_msg("%s, line %d: ignoring unknown identifier '%s'",
+				cfname, lnum, lv);
+			file_needs_update = true;
+			continue;
+		}
+
+		lp += strspn(lp, " ");
+		if (*lp++ != '=')
+			err_quit("parse error in %s, line %d: missing '=' operator in assignment",
+				 cfname, lnum);
+		lp += strspn(lp, " ");
+
+		len = strcspn(lp, " \n");
+		if (len > sizeof(rv))
+			err_quit("parse error in %s, line %d: argument too long", cfname, lnum);
+		else if (*lp == '\n')
+			err_quit("parse error in %s, line %d: argument expected", cfname, lnum);
+		strncpy(rv, lp, len);
+		rv[len] = '\0';
+
+		switch (ci->type) {
+		case t_int:
+			v_int = strtol(rv, &conv_err, 10);
+			if (*conv_err != '\0') {
+				err_quit("parse error in %s, line %d: integer value expected, '%s' found instead",
+					 cfname, lnum, rv);
+			} else if (v_int > ci->max) {
+				err_msg("%s, line %d: value exceeds maximum of %d - using maximum",
+					 cfname, lnum, (int)ci->max);
+				*ci->v.i = ci->max;
+				file_needs_update = true;
+			} else if (v_int < ci->min) {
+				err_msg("%s, line %d: value is below minimum of %d - using minimum",
+					 cfname, lnum, (int)ci->min);
+				*ci->v.i = ci->min;
+				file_needs_update = true;
+			} else {
+				*ci->v.i = v_int;
+			}
+			break;
+		case t_list:
+			assert(ci->list != NULL);
+			if (!argv_count(ci->list))
+				err_quit("no usable %s candidates available for '%s'", ci->name, rv);
+			v_int = argv_find(ci->list, rv);
+			if (v_int < 0) {
+				err_msg("%s, line %d: '%s = %s' is not valid - using defaults",
+					 cfname, lnum, lv, rv);
+				file_needs_update = true;
+			} else {
+				*ci->v.i = v_int;
+			}
+		case t_sep:
+		case t_func:
+			break;
+		}
+	}
+	fclose(fd);
+done:
+	free(cfname);
+
+	if (file_needs_update) {
+		write_cf();
+	}
+}
+
 static void init_conf_items(void)
 {
 	struct conf_item *item;
@@ -305,12 +316,35 @@ static void init_conf_items(void)
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Interface");
 	item->cfname	= strdup("interface");
-	item->type	= t_listval;
-	item->v.s	= conf.ifname;
-	item->max	= 10;
+	item->type	= t_list;
+	item->v.i	= &conf.if_idx;
 	item->list	= if_list;
 	ll_push(conf_items, "*", item);
 
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Cisco-style MAC addresses");
+	item->cfname	= strdup("cisco_mac");
+	item->type	= t_list;
+	item->v.i	= &conf.cisco_mac;
+	item->list	= on_off_names;
+	ll_push(conf_items, "*", item);
+
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Scan sort type");
+	item->cfname	= strdup("sort_order");
+	item->type	= t_list;
+	item->v.i	= &conf.scan_sort_order;
+	item->list	= sort_order;
+	ll_push(conf_items, "*", item);
+
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Scan sort in ascending order");
+	item->cfname	= strdup("sort_ascending");
+	item->type	= t_list;
+	item->v.i	= &conf.scan_sort_asc;
+	item->list	= on_off_names;
+	ll_push(conf_items, "*", item);
+
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Statistics updates");
 	item->cfname	= strdup("stat_updates");
@@ -367,8 +401,9 @@ static void init_conf_items(void)
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Override scale autodetect");
 	item->cfname	= strdup("override_auto_scale");
-	item->type	= t_switch;
-	item->v.b	= &conf.override_bounds;
+	item->type	= t_list;
+	item->v.i	= &conf.override_bounds;
+	item->list	= on_off_names;
 	ll_push(conf_items, "*", item);
 
 	item = calloc(1, sizeof(*item));
@@ -376,8 +411,8 @@ static void init_conf_items(void)
 	item->cfname	= strdup("min_signal_level");
 	item->type	= t_int;
 	item->v.i	= &conf.sig_min;
-	item->min	= -128;
-	item->max	= -60;
+	item->min	= -100;
+	item->max	= -39;
 	item->inc	= 1;
 	item->unit	= strdup("dBm");
 	item->dep	= &conf.override_bounds;
@@ -388,8 +423,8 @@ static void init_conf_items(void)
 	item->cfname	= strdup("max_signal_level");
 	item->type	= t_int;
 	item->v.i	= &conf.sig_max;
-	item->min	= -59;
-	item->max	= 120;
+	item->min	= -40;
+	item->max	= -10;
 	item->inc	= 1;
 	item->unit	= strdup("dBm");
 	item->dep	= &conf.override_bounds;
@@ -400,8 +435,8 @@ static void init_conf_items(void)
 	item->cfname	= strdup("min_noise_level");
 	item->type	= t_int;
 	item->v.i	= &conf.noise_min;
-	item->min	= -128;
-	item->max	= -60;
+	item->min	= -120;
+	item->max	= -70;
 	item->inc	= 1;
 	item->unit	= strdup("dBm");
 	item->dep	= &conf.override_bounds;
@@ -412,31 +447,20 @@ static void init_conf_items(void)
 	item->cfname	= strdup("max_noise_level");
 	item->type	= t_int;
 	item->v.i	= &conf.noise_max;
-	item->min	= -60;
-	item->max	= 120;
+	item->min	= -69;
+	item->max	= -40;
 	item->inc	= 1;
 	item->unit	= strdup("dBm");
 	item->dep	= &conf.override_bounds;
 	ll_push(conf_items, "*", item);
 
-	item = calloc(1, sizeof(*item));
-	item->name	= strdup("Random signals");
-	item->cfname	= strdup("random");
-	item->type	= t_switch;
-	item->v.b	= &conf.random;
-	ll_push(conf_items, "*", item);
-
 	/* thresholds */
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Low threshold action");
 	item->cfname	= strdup("lo_threshold_action");
 	item->type	= t_list;
-	item->v.b	= &conf.lthreshold_action;
-	item->list	= ll_create();
-	ll_push(item->list, "s", "Disabled");
-	ll_push(item->list, "s", "Beep");
-	ll_push(item->list, "s", "Flash");
-	ll_push(item->list, "s", "Beep+Flash");
+	item->v.i	= &conf.lthreshold_action;
+	item->list	= action_items;
 	ll_push(conf_items, "*", item);
 
 	item = calloc(1, sizeof(*item));
@@ -455,12 +479,8 @@ static void init_conf_items(void)
 	item->name	= strdup("High threshold action");
 	item->cfname	= strdup("hi_threshold_action");
 	item->type	= t_list;
-	item->v.b	= &conf.hthreshold_action;
-	item->list	= ll_create();
-	ll_push(item->list, "s", "Disabled");
-	ll_push(item->list, "s", "Beep");
-	ll_push(item->list, "s", "Flash");
-	ll_push(item->list, "s", "Beep+Flash");
+	item->v.i	= &conf.hthreshold_action;
+	item->list	= action_items;
 	ll_push(conf_items, "*", item);
 
 	item = calloc(1, sizeof(*item));
@@ -489,11 +509,8 @@ static void init_conf_items(void)
 	item->name	= strdup("Startup screen");
 	item->cfname	= strdup("startup_screen");
 	item->type	= t_list;
-	item->v.b	= &conf.startup_scr;
-	item->list	= ll_create();
-	ll_push(item->list, "s", "Info");
-	ll_push(item->list, "s", "Histogram");
-	ll_push(item->list, "s", "Access points");
+	item->v.i	= &conf.startup_scr;
+	item->list	= screen_names;
 	ll_push(conf_items, "*", item);
 
 	/* separator (dummy entry) */
@@ -509,37 +526,49 @@ static void init_conf_items(void)
 	ll_push(conf_items, "*", item);
 }
 
-static void set_defaults(void)
-{
-	strncpy(conf.ifname, ll_get(if_list, 0), sizeof(conf.ifname));
-
-	conf.stat_iv		= 100;
-	conf.info_iv		= 10;
-	conf.slotsize		= 4;
-	conf.meter_decay	= 0;
-
-	conf.override_bounds	= false;
-	conf.random		= false;
-
-	conf.sig_min		= -102;
-	conf.sig_max		= 10;
-	conf.noise_min		= -102;
-	conf.noise_max		= 10;
-
-
-	conf.lthreshold_action	= TA_DISABLED;
-	conf.lthreshold		= -80;
-	conf.hthreshold_action	= TA_DISABLED;
-	conf.hthreshold		= -10;
-
-	conf.startup_scr	= 0;
-}
-
 void getconf(int argc, char *argv[])
 {
-	if_list = iw_get_interface_list();
-	set_defaults();
+	int arg, help = 0, version = 0;
+
+	conf_get_interface_list();
 	init_conf_items();
 	read_cf();
-	getargs(argc, argv);
+
+	while ((arg = getopt(argc, argv, "ghi:v")) >= 0) {
+		switch (arg) {
+		case 'g':
+			conf.check_geometry = true;
+			break;
+		case 'h':
+			help++;
+			break;
+		case 'i':
+			conf.if_idx = argv_find(if_list, optarg);
+			if (conf.if_idx < 0)
+				err_quit("no wireless extensions found on '%s'",
+					 optarg);
+			break;
+		case 'v':
+			version++;
+			break;
+		default:
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (version) {
+		printf("wavemon %s", PACKAGE_VERSION);
+		printf(" with %s.\n", curses_version());
+		printf("Distributed under the terms of the GPLv3.\n%s", help ? "\n" : "");
+	}
+	if (help) {
+		printf("usage: wavemon [ -hgv ] [ -i ifname ]\n");
+		printf("  -g            Ensure screen is sufficiently dimensioned\n");
+		printf("  -h            This help screen\n");
+		printf("  -i <ifname>   Use specified network interface (default: auto)\n");
+		printf("  -v            Print version number\n");
+	}
+
+	if (version || help)
+		exit(EXIT_SUCCESS);
 }
