@@ -70,11 +70,11 @@ void waddstr_center(WINDOW *win, int y, const char *s)
 	mvwaddstr(win, y, (WAV_WIDTH - strlen(s)) / 2, s);
 }
 
-void waddstr_b(WINDOW *win, const char *s)
+void wadd_attr_str(WINDOW *win, const int attrs, const char *s)
 {
-	wattron(win, A_BOLD);
+	wattron(win, attrs);
 	waddstr(win, s);
-	wattroff(win, A_BOLD);
+	wattroff(win, attrs);
 }
 
 /* Enforce that @str is at most @len characters (excluding the terminal '\0') */
@@ -84,12 +84,16 @@ const char *curtail(const char *str, const char *sep, int len)
 	const char fallback_sep[] = "~";
 	int l = 0, front, mid, back;
 
-	assert(len < sizeof(out_buf));
+	if (len >= sizeof(out_buf))
+		len = sizeof(out_buf) - 1;
 
-	if (sep == NULL)
+	if (sep == NULL || *sep == '\0')
 		sep = fallback_sep;
 	mid = strlen(sep);
-	assert(len >= mid);
+	if (mid > len) {
+		sep = fallback_sep;
+		mid = strlen(sep);
+	}
 
 	if (str != NULL)
 		l = strlen(str);
@@ -113,7 +117,7 @@ static double interpolate(const double val, const double min, const double max)
 }
 
 void waddbar(WINDOW *win, int y, float v, float min, float max,
-		    char *cscale, bool rev)
+		    int8_t *cscale, bool rev)
 {
 	chtype ch = '=' | A_BOLD | cp_from_scale(v, cscale, rev);
 	int len = MAXXLEN * interpolate(v, min, max);
@@ -123,7 +127,7 @@ void waddbar(WINDOW *win, int y, float v, float min, float max,
 }
 
 void waddthreshold(WINDOW *win, int y, float v, float tv,
-		   float minv, float maxv, char *cscale, chtype tch)
+		   float minv, float maxv, int8_t *cscale, chtype tch)
 {
 	if (tv > minv && tv < maxv) {
 		if (v > tv)
